{ This application makes creating aliases easy and convenient. }
{}
{ The user drag-drops a file onto Easy Alias, and an alias of that file with the same name is created }
{ next to the application, usually on the desktop.}
{ If the user just double-clicks on Easy Alias, she gets a standard get-file dialog.}
{}
{ NOTE: Aliases made in this manner will be automatically updated by the Finder whenever }
{ the original file moves. }
{}
{ by Allen H Simon 10/1/91 AOL: Wahoo Five }
{ based on AliasMaker program}
{ by Joe Zobkiw 6/12/91 AOL: AFL Zobkiw }
{}

program AliasMaker;
	uses
		Types, OSUtils, Files, Aliases;

	const
		kSystem7 = $0700;					{ system 7.0 }
		kAliasResourceID = 0;				{ resource ID of alis resource }
		kIsAliasFlag = 32768;				{ to set Finder flag }
		InitedFlag = $0100;					{ flags finder that location is set}
		OffsetDistance = 60;					{ distance to offset icon in pixels}
		Margin = 25;							{ this is how close to the edge of the screen the new icon can be}

	var
		MyVolNumber: integer;				{ gets volume number of this program}
		myLocation: point;					{ find my location on desktop (or in window)}
		CurrTicks: longint;					{ ignored var parameter to Delay}

{____________________________________________________________}
{ This routine initializes the Macintosh Toolbox }
{____________________________________________________________}
	procedure InitMacintosh;
	begin
{$IFC not THINK_PASCAL}
{these are predefined in Think Pascal}
		InitGraf(@thePort);
		InitFonts;
		InitWindows;
		InitMenus;
		TEInit;
		InitDialogs(nil);
{$ENDC}
		FlushEvents(EveryEvent, 0);
		InitCursor;
	end;

{____________________________________________________________}
{ make sure we can run, we should really check Gestalt here }
{____________________________________________________________}
	procedure CheckEnvironment;
		var
			env: SysEnvRec;
			err: OSerr;
	begin
		err := SysEnvirons(1, env);
		if (err <> noErr) then
			begin
				SysBeep(1);
				Delay(60, CurrTicks);
				ExitToShell;
			end;
		;
		if (env.systemVersion < kSystem7) then
			begin
				SysBeep(1);
				Delay(60, CurrTicks);
				ExitToShell;
			end;
	end;

{____________________________________________________________}
{ get Finder information on startup; if none, call SFGetFile }
{____________________________________________________________}
	procedure GetFileToAlias (var reply: SFReply; var spec: FSSpec);
		var
			NumberOfFiles, ignore: integer;
			FileInfo: AppFile;
			parDirID: longint;
			procID: longint;
			err: OSerr;
			where: Point;
			typeList: SFTypeList;
			vRefNum: integer;

	begin
		SetPt(where, 50, 50);
		CountAppFiles(ignore, NumberOfFiles);
		if NumberOfFiles = 0 then
			SFGetFile(where, '', nil, -1, typeList, nil, reply)
		else
			begin
				GetAppFiles(1, FileInfo);		{only gets the first one}
				with reply do
					begin
						good := true;
						copy := false;
						fType := FileInfo.Ftype;
						vRefNum := FileInfo.vRefNum;
						version := FileInfo.versNum;
						fName := FileInfo.fName;
					end;
			end;

		if reply.good then
			begin
		{ get some working directory information for the original file }
				err := GetWDInfo(reply.vRefNum, vRefNum, parDirID, procID);
				if (err <> noErr) then
					begin
						SysBeep(1);
						Delay(60, CurrTicks);
						ExitToShell;
					end;

		{ fill in the spec record now }
				spec.vRefNum := vRefNum;
				spec.parID := parDirID;
				spec.name := reply.fName;

			end;
	end;

{____________________________________________________________}
{ create an alias filethis is the core of this program! }
{____________________________________________________________}
	function CreateAlias (originalReply: SFReply; aliasReply: SFReply; spec: FSSpec): OSErr;
		var
			err: OSErr;
			myAliasHandle: AliasHandle;
			aliasFileRefNum: integer;
			saveResFile: integer;
			fndrInfo: FInfo;
			vRefNum: integer;
			parDirID: longint;
			procID: longint;
			newLocation: point;

	{____________________________________________________________}
		procedure Fail (err: OSErr);
		begin
			if (err <> noErr) then
				begin
					CreateAlias := err;

					UseResFile(saveResFile);

					if myAliasHandle <> nil then
						DisposHandle(Handle(myAliasHandle));

					Exit(CreateAlias);
				end;
		end;

{____________________________________________________________}
	begin	{CreateAlias}
		CreateAlias := noErr;

	{ save our current file }
		saveResFile := CurResFile;

	{ create the new alias }
		err := NewAlias(nil, spec, myAliasHandle);
		Fail(err);

	{ check the handle for nil }
		if (myAliasHandle = nil) then
			Fail(memFullErr);

	{ get some working directory information for the alias file }
		err := GetWDInfo(aliasReply.vRefNum, vRefNum, parDirID, procID);
		Fail(err);

	{ create the alias file tech note 241}
		HCreateResFile(aliasReply.vRefNum, parDirID, aliasReply.fName);
		Fail(ResError);
	{ N.B. won't create a duplicate file name}

	{ open the alias file }
		aliasFileRefNum := HOpenResFile(aliasReply.vRefNum, parDirID, aliasReply.fName, fsWrPerm);
		if (aliasFileRefNum = -1) then
			Fail(aliasFileRefNum);

	{ use the alias resource file }
		UseResFile(aliasFileRefNum);
		Fail(ResError);

	{ add a resource for the alias of type 'alis' }
		AddResource(Handle(myAliasHandle), rAliasType, kAliasResourceID, aliasReply.fName);
		Fail(ResError);

	{ write it out }
		WriteResource(Handle(myAliasHandle));
		Fail(ResError);

	{ close the new alias }
		CloseResFile(aliasFileRefNum);
		Fail(ResError);

	{ we need the creator of the original file }
		err := HGetFInfo(originalReply.vRefNum, spec.parID, originalReply.fName, fndrInfo);
		Fail(err);

	{ set it up as an alias, and flag that it's location is already determined }
		fndrInfo.fdFlags := kIsAliasFlag + InitedFlag;

	{ set the new location in relation to application location}
		newLocation := myLocation;
		if newLocation.h + OffsetDistance < screenbits.bounds.right - Margin then		{if there's room on the right...}
			newLocation.h := newLocation.h + OffsetDistance										{then put to the right}
		else if newLocation.v + OffsetDistance < screenbits.bounds.bottom - Margin then{else if there's room below...}
			newLocation.v := newLocation.v + OffsetDistance										{then put below}
		else
			newLocation.v := newLocation.v - OffsetDistance;									{else put above}
		fndrInfo.fdLocation := newLocation;

	{ set the info of the new file }
		err := HSetFInfo(aliasReply.vRefNum, parDirID, aliasReply.fName, fndrInfo);
		Fail(err);

	 { restore our file }
		UseResFile(saveResFile);
		Fail(ResError);

	end;	{CreateAlias}

{____________________________________________________________}
{ this is the main program }
{____________________________________________________________}
	var

		err: OSErr;
		originalReply: SFReply;
		aliasReply: SFReply;
		spec: FSSpec;
		finderInfo: FInfo;
		myName: Str255;
		ignore: integer;
		garbage: Handle;

begin

	InitMacintosh;
	CheckEnvironment;

	err := GetVol(nil, MyVolNumber);		{must get my volume first, before possible calling SFGetFile}
	if err <> NoErr then
		begin
			SysBeep(1);
			Delay(60, CurrTicks);
			ExitToShell;
		end;

	GetAppParms(myName, ignore, garbage);
	err := GetFInfo(myName, myVolNumber, finderInfo);
	if err <> NoErr then
		begin
			SysBeep(1);
			Delay(60, CurrTicks);
			ExitToShell;
		end;
	myLocation := finderInfo.fdLocation;		{here's my location on desktop or in window}

	GetFileToAlias(originalReply, spec);
	if (not originalReply.good) then
		ExitToShell;

	AliasReply := OriginalReply;
	aliasReply.vRefNum := myVolNumber;
	err := CreateAlias(originalReply, aliasReply, spec);
	if err <> NoErr then
		begin
			SysBeep(1);
			Delay(60, CurrTicks);
		end;

end.